#!/usr/bin/env python
#
#       exallpoly.py
#       
#       Copyright 2010 Chris Hill <>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import sys
import os.path
from optparse import OptionParser
import shpUtils

def main():
    parser = OptionParser(usage="%prog [options] shapefile", version="%prog 1.0")
    #parser.add_option("-p", "--polygon", dest="polyname",help="the polygon name to extract")
    parser.add_option("-b", "--bounds",action="store_true", dest="bounds", default=False,help="add bounds statement")
    parser.add_option("-q", "--quiet",action="store_false", dest="verbose", default=True,help="don't print status messages to stdout")
    parser.add_option("-n", "--nodeid",action="store", dest="nodeid",type="int", default=-1, help="The first nodeid to use, it should be negative")
    parser.add_option("-w", "--wayid",action="store", dest="wayid", type="int", default=-1, help="The first wayid to use, it should be negative")
    parser.add_option("-o", "--outfile",action="store",dest="osmfile", help="The output file name")
    (options, args) = parser.parse_args()
    
    if len(args) != 1 :
        print "You must enter the shapefilename"
        sys.exit(1)
    
    shapefilename=args[0] # the left over stuff when the options have been extracted
    
    if  not os.path.exists(shapefilename) :
        print "The shpfile %s cannot be found" % options.shpfile
        sys.exit(1)
    
    #if options.polyname is None :
    #    print "You must supply the polygon name to extract"
    #    sys.exit(1)
    
    startnodeid=options.nodeid
    if startnodeid>=0 :
        print "Nodeid must be a negative number"
        sys.exit(1)
    
    startwayid=options.wayid
    if startwayid>=0 :
        print "Wayid must be a negative number"
        sys.exit(1)
    
    if options.osmfile is None :
        osmfile="polys.osm"
    else :
        osmfile=options.osmfile
    
    if options.verbose :
        print "ExtractPoly - extract a named polygon from a shapefile"
    
    #polyname=options.polyname

    if options.verbose :
        print "Loading the shapefile: "+shapefilename
    # load the shapefile, populating a list of dictionaries
    shpRecords = shpUtils.loadShapefile(shapefilename)

    # open the outpu file ready for writing to
    pf = file(osmfile,"w")
    pf.write("<?xml version='1.0' encoding='UTF-8'?>\n")
    pf.write("<osm version='0.6' generator='extractpoly'>\n")
    
    #set the nodeid & wayid
    nodeid=startnodeid
    wayid=startwayid
    
    #work through the records, extracting each polygon
    for poly in shpRecords :
        #pname=poly['dbf_data']['NAME'].rstrip()
        #if pname==polyname :
        
        waynodeid=nodeid
        
        # extract the polygon details to a file
        for pt in poly['shp_data']['parts'][0]['points'] :
            #for every pair of points write a node
            lat=pt['y']
            lon=pt['x']
            pf.write("  <node id='%d' visible='true' lat='%0.9f' lon='%0.9f' />\n" % (nodeid,lat,lon))
            nodeid-=1
        
        #add the way to add
        
        pf.write("  <way id='%d' action='modify' visible='true'>\n" % (wayid))
        wayid-=1 #ready for the next way
        
        #add the list of nodes to the way, but not the last one
        # the last one would be the same as the first in a shapefile polygon
        # so in OSM just reuse the first node to avoid duplicate nodes
        for i in range(waynodeid,nodeid+1,-1) :
            pf.write("    <nd ref='%d' />\n" % (i))
        
        # close the way by adding the first node again (the OSM way to do it)
        pf.write("    <nd ref='%d' />\n" % (waynodeid))
        
        # add the tags for the way
        pf.write("    <tag k='natural' v='water' />\n")
        pf.write("    <tag k='source' v='OS_Vectormap' />\n")
        
        # end the way
        pf.write("  </way>\n")
        
    #write finish off the file and close it
    pf.write("</osm>\n")
    del pf
    
    return 0

if __name__ == '__main__': main()
